﻿-- Tidy Bar
local TidyBarScale = 1

local MenuButtonFrames = {
	CharacterMicroButton,
	SpellbookMicroButton,
	TalentMicroButton,
	AchievementMicroButton,
	QuestLogMicroButton,
	GuildMicroButton,
	PVPMicroButton,
	LFDMicroButton,
	MainMenuMicroButton,
	HelpMicroButton,
}

local BagButtonFrames = {
	MainMenuBarBackpackButton,
	CharacterBag0Slot,
	CharacterBag1Slot,
	CharacterBag2Slot,
	CharacterBag3Slot,
	KeyRingButton,
}

local isButtonGridShown = false
local prevAlpha = 0
local initTime = 0
local Corner_Artwork_Texture = "Interface\\Addons\\TidyBar\\CornerArt"
local Empty_Art = "Interface\\Addons\\TidyBar\\Empty"

local TidyBar = CreateFrame("Frame", "TidyBar", UIParent)
local CornerMenuFrame = CreateFrame("Frame", nil, UIParent)

CornerMenuFrame.Texture = CornerMenuFrame:CreateTexture(nil,"BACKGROUND")
CornerMenuFrame.Texture:SetTexture(Corner_Artwork_Texture)
CornerMenuFrame.MicroButtons = CreateFrame("Frame", nil, CornerMenuFrame)
CornerMenuFrame.BagButtonFrames = CreateFrame("Frame", nil, CornerMenuFrame)

function SetTransparent(frame) 
		frame:Hide()
		frame:SetAlpha(0)
end

function UpdateActionBars()
	local anchor
	local anchorOffset = 4
	local repOffset = 0
	
	if MainMenuExpBar:IsShown() then repOffset = 9 end
	if ReputationWatchBar:IsShown() then repOffset = repOffset + 9 end
		
	if MultiBarBottomLeft:IsShown() then
		anchor = MultiBarBottomLeft
		anchorOffset = 4
	else
		anchor = ActionButton1;
		anchorOffset = 8 + repOffset
	end

    if MultiBarBottomRight:IsShown() then
		MultiBarBottomRight:ClearAllPoints()
		MultiBarBottomRight:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, anchorOffset )
		anchor = MultiBarBottomRight
		anchorOffset = 4
	end

	if ShapeshiftButton1:IsShown() then
		ShapeshiftButton1:ClearAllPoints();
		ShapeshiftButton1:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, anchorOffset);
		anchor = ShapeshiftButton1
		anchorOffset = 4
	end
	
	if MultiCastActionBarFrame:IsShown() then	-- Totem bar
		MultiCastActionBarFrame:ClearAllPoints();
		MultiCastActionBarFrame:_SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, anchorOffset);
		anchor = MultiCastActionBarFrame
		anchorOffset = 4
	end

	PetActionButton1:ClearAllPoints()
	PetActionButton1:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, anchorOffset)
	anchor = PetActionButton1
	anchorOffset = 4

	PossessButton1:ClearAllPoints();
	PossessButton1:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, anchorOffset);
end

local function UpdateCorner()
	-- Main Menu
    CharacterMicroButton:ClearAllPoints();
    CharacterMicroButton:SetPoint("BOTTOMRIGHT", CornerMenuFrame.MicroButtons, "BOTTOMRIGHT", -226, 0);

	-- Bag Buttons
    MainMenuBarBackpackButton:ClearAllPoints();
	MainMenuBarBackpackButton:SetPoint("BOTTOM");
	MainMenuBarBackpackButton:SetPoint("RIGHT", -38, 0);
	KeyRingButton:ClearAllPoints();
	KeyRingButton:SetPoint("BOTTOM");
    KeyRingButton:SetPoint("LEFT", MainMenuBarBackpackButton, "RIGHT", 16, -4 )
end

local function UpdateUI()
	if InCombatLockdown() then return end 
	-- Hide backgrounds
	SetTransparent(SlidingActionBarTexture0)
	SetTransparent(SlidingActionBarTexture1)
    SetTransparent(ShapeshiftBarLeft)
    SetTransparent(ShapeshiftBarMiddle)
    SetTransparent(ShapeshiftBarRight)
    SetTransparent(PossessBackground1)
    SetTransparent(PossessBackground2)

    UpdateActionBars()
    UpdateCorner()

	-- Scaling
	MainMenuBar:SetScale(TidyBarScale)
	MultiBarBottomRight:SetScale(TidyBarScale)
	MultiBarBottomLeft:SetScale(TidyBarScale)
	MultiBarRight:SetScale(TidyBarScale)
	MultiBarLeft:SetScale(TidyBarScale)
end

local function FadeSideBar(desired)
	sideBarAlpha = desired
	if isButtonGridShown then final = 1 else final = desired end
	for i = 1, 12 do
		_G["MultiBarRightButton"..i]:SetAlpha(final);
		_G["MultiBarLeftButton"..i]:SetAlpha(final);
	end
end

local function FadeCorner(Alpha)
	if UnitHasVehicleUI("player") then 
		Alpha = 1
		CornerMenuFrame:SetAlpha(0)
	else CornerMenuFrame:SetAlpha(Alpha) end

	for i, name in pairs(BagButtonFrames) do name:SetAlpha(Alpha) end
end

local function SetMouseOverCornerButton(frameTarget)
	frameTarget:HookScript("OnEnter", function() FadeCorner(1)   end)
	frameTarget:HookScript("OnLeave", function() FadeCorner(0) end)
end

local function SetMouseOverCornerBars()
	local CornerMouseoverFrame = CreateFrame("Frame", "BagBarMouseoverBox", UIParent)
	CornerMouseoverFrame:EnableMouse();
	CornerMouseoverFrame:HookScript("OnEnter", function() FadeCorner(1) end)
	CornerMouseoverFrame:HookScript("OnLeave", function() FadeCorner(0) end)
	CornerMouseoverFrame:SetFrameStrata("BACKGROUND")
	
	CornerMouseoverFrame:SetPoint("TOP", MainMenuBarBackpackButton, "TOP", 0,10)
	CornerMouseoverFrame:SetPoint("RIGHT", KeyRingButton, "RIGHT", 4,0)
	CornerMouseoverFrame:SetPoint("LEFT", CharacterMicroButton, "LEFT", -20,0)
	
	KeyRingButton:Enable();
    KeyRingButton:EnableDrawLayer();
    KeyRingButton:Show();
	
	for i, name in pairs(MenuButtonFrames) do SetMouseOverCornerButton( name) end
	for i, name in pairs(BagButtonFrames) do SetMouseOverCornerButton( name) end
end

local function SetMouseOverSideButton(frameTarget)
	frameTarget:HookScript("OnEnter", function() FadeSideBar(1) end)
	frameTarget:HookScript("OnLeave", function() FadeSideBar(0) end)
end

local function SetMouseOverSideBars()
	local SideMouseoverFrame = CreateFrame("Frame", "SideBarMouseoverBox", UIParent)
	
	SideMouseoverFrame:SetPoint("TOPLEFT", MultiBarLeft, "TOPLEFT", -10,0)
	SideMouseoverFrame:SetPoint("BOTTOMRIGHT", MultiBarRight, "BOTTOMRIGHT", 0,0)
	
	SideMouseoverFrame:EnableMouse();
	SideMouseoverFrame:HookScript("OnEnter", function() FadeSideBar(1) end)
	SideMouseoverFrame:HookScript("OnLeave", function() FadeSideBar(0) end)
	
	MultiBarRight:EnableMouse();
	MultiBarRight:HookScript("OnEnter", function() FadeSideBar(1) end)
	MultiBarRight:HookScript("OnLeave", function() FadeSideBar(0) end)

	MultiBarLeft:EnableMouse();
	MultiBarLeft:HookScript("OnEnter", function() FadeSideBar(1) end)
	MultiBarLeft:HookScript("OnLeave", function() FadeSideBar(0) end)
	
	for i = 1, 12 do
		SetMouseOverSideButton( _G["MultiBarRightButton"..i] )
		SetMouseOverSideButton( _G["MultiBarLeftButton"..i] )
	end
	
end

local function Startup()
	-- Call Update Function when the default UI makes changes
	hooksecurefunc("UIParent_ManageFramePositions", UpdateUI);
	-- Required in order to move the frames around
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomRight"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["PetActionBarFrame"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["ShapeshiftBarFrame"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["PossessBarFrame"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiCastActionBarFrame"] = nil
	
	-- Change the size of the central button and status bars
    MainMenuBar:SetWidth(512);
	MainMenuExpBar:SetWidth(512);
    ReputationWatchBar:SetWidth(512);
    MainMenuBarMaxLevelBar:SetWidth(512);
    ReputationWatchStatusBar:SetWidth(512);
	--ReputationWatchBar:SetWidth(512)

	-- Adjust the fill and endcap artwork
	MainMenuBarTexture0:SetPoint("LEFT", MainMenuBar, "LEFT", 0, 0);
    MainMenuBarTexture1:SetPoint("RIGHT", MainMenuBar, "RIGHT", 0, 0);
 	MainMenuBarLeftEndCap:SetPoint("RIGHT", MainMenuBar, "LEFT", 32, 0);
    MainMenuBarRightEndCap:SetPoint("LEFT", MainMenuBar, "RIGHT", -32, 0); 
	
	-- Hide Unwanted Art
	MainMenuBarPageNumber:Hide();
    ActionBarUpButton:Hide();
    ActionBarDownButton:Hide();
	-- Experience Bar
	MainMenuBarTexture2:SetTexture(Empty_Art)
	MainMenuBarTexture3:SetTexture(Empty_Art)
	MainMenuBarTexture2:SetAlpha(0)
	MainMenuBarTexture3:SetAlpha(0)
	for i=1,19 do _G["MainMenuXPBarDiv"..i]:SetTexture(Empty_Art) end
	-- Max-level Rep Bar
	MainMenuMaxLevelBar0:SetAlpha(0)
	MainMenuMaxLevelBar1:SetAlpha(0)
	MainMenuMaxLevelBar2:SetAlpha(0)
	MainMenuMaxLevelBar3:SetAlpha(0)
	-- Rep Bar Bubbles (For the Rep Bar)
	ReputationWatchBarTexture0:SetAlpha(0)
	ReputationWatchBarTexture1:SetAlpha(0)
	ReputationWatchBarTexture2:SetAlpha(0)
	ReputationWatchBarTexture3:SetAlpha(0)
	-- Rep Bar Bubbles (for the XP bar)
	ReputationXPBarTexture0:SetAlpha(0)
	ReputationXPBarTexture1:SetAlpha(0)
	ReputationXPBarTexture2:SetAlpha(0)
	ReputationXPBarTexture3:SetAlpha(0)
		
	-- Setup the Corner Button Frames
	CornerMenuFrame:SetFrameStrata("LOW")
	CornerMenuFrame:SetWidth(256) -- Set these to whatever height/width is needed 
	CornerMenuFrame:SetHeight(128) -- for your Texture
	CornerMenuFrame:SetPoint("BOTTOMRIGHT")
	CornerMenuFrame:SetScale(1.09*TidyBarScale)
	CornerMenuFrame.Texture:SetAllPoints(CornerMenuFrame)
	CornerMenuFrame.MicroButtons:SetAllPoints(CornerMenuFrame)
	CornerMenuFrame.BagButtonFrames:SetPoint("RIGHT")
	CornerMenuFrame.BagButtonFrames:SetPoint("TOP", -8, -24)
	CornerMenuFrame.BagButtonFrames:SetHeight(64)
	CornerMenuFrame.BagButtonFrames:SetWidth(256)
	
	for i, name in pairs(MenuButtonFrames) do
		name:SetParent(CornerMenuFrame.MicroButtons)
		name:SetScale(1/1.09)
	end
	
	for i, name in pairs(BagButtonFrames) do
		name:SetParent(CornerMenuFrame.BagButtonFrames)
		name:SetScale(1/1.09)
	end
	
	KeyRingButton:SetParent(CornerMenuFrame.BagButtonFrames)
	KeyRingButton:SetScale(.75)
	MainMenuBarBackpackButton:SetScale(.75)
	
	-- Set Pet Bars
	PetActionBarFrame:SetAttribute("unit", "pet")
	RegisterUnitWatch(PetActionBarFrame)

	-- Set Mouseover for Right-side Button Bar
	SetMouseOverSideBars()
	FadeSideBar(0)
	SetMouseOverCornerBars()
	FadeCorner(0)

	MultiCastActionBarFrame._SetPoint = MultiCastActionBarFrame.SetPoint
	MultiCastActionBarFrame.SetPoint = function() UpdateUI() end
end
	
-- Event Handlers
local events = {}

function events:ACTIONBAR_SHOWGRID() isButtonGridShown = true; FadeSideBar(sideBarAlpha) end
function events:ACTIONBAR_HIDEGRID() isButtonGridShown = false; FadeSideBar(sideBarAlpha) end
function events:UNIT_ENTERED_VEHICLE() UpdateUI(); FadeCorner(1) end
function events:UNIT_EXITED_VEHICLE() UpdateUI(); FadeCorner(0) end
function events:PLAYER_ENTERING_WORLD() Startup(); FadeCorner(0) end
function events:UPDATE_INSTANCE_INFO() UpdateUI(); end
function events:SPELL_UPDATE_USEABLE() UpdateUI(); end
function events:UPDATE_BONUS_ACTIONBAR() UpdateUI(); end
function events:UPDATE_MULTI_CAST_ACTIONBAR() UpdateUI(); end
function events:PET_BAR_UPDATE() UpdateUI();end
function events:CLOSE_WORLD_MAP() UpdateUI();end

local function EventHandler(frame, event) if events[event] then events[event]() end end

-- Set Event Monitoring
for eventname in pairs(events) do 
	TidyBar:RegisterEvent(eventname)
end

TidyBar:SetScript("OnEvent", EventHandler);
TidyBar:Show()
